from pydantic import BaseModel, Field


class BalanceDetails(BaseModel):
    total: float = Field(...)
    available: float = Field(...)
    deposition_pending: float | None = Field(None)
    blocked: float | None = Field(None)
    debt: float | None = Field(None)
    hold: float | None = Field(None)


class LinkedCard(BaseModel):
    pan_fragment: str = Field(...)
    card_type: str = Field(None, alias="type")


class AccountInfo(BaseModel):
    """
    Получение информации о состоянии счета пользователя
    https://yoomoney.ru/docs/wallet/user-account/account-info
    """
    account: str = Field(...)  # номер счета
    balance: float = Field(...)  # баланс счета
    currency: str = Field(...)  # код валюты счета
    account_type: str = Field(...)
    account_status: str = Field(...)
    balance_details: BalanceDetails | None = Field(None)
    cards_linked: list[LinkedCard] | None = Field(None)



from typing import Literal

from httpx import Client, AsyncClient, Response

from ayoomoney.errors import CreateTokenError

AUTH_URL = "https://yoomoney.ru/oauth/authorize"
TOKEN_URL = "https://yoomoney.ru/oauth/token"
DEFAULT_SCOPE = (
    "account-info",
    "operation-history",
    "operation-details",
    "incoming-transfers",
    "payment-p2p",
    "payment-shop",
)


class _BaseAuthorization:
    def __init__(self, client_id: str, redirect_uri: str):
        self.client_id = client_id
        self.redirect_uri = redirect_uri

    def _process_get_access_token(self, response: Response) -> str | None:
        data = response.json()
        access_token = data.get("access_token", None)
        error = data.get("error", None)
        if error or not access_token:
            if not access_token:
                raise CreateTokenError(error="invalid_request")
            raise CreateTokenError(error=error)
        return access_token


class Authorization(_BaseAuthorization):
    def __init__(self, client_id: str, redirect_uri: str):
        super().__init__(client_id, redirect_uri)
        self.client = Client(
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            }
        )

    def close(self):
        self.close()

    def authorization_request(
            self,
            response_type: Literal["code"] = "code",
            scope: list[str] = DEFAULT_SCOPE,
            instance_name: str | None = None
    ) -> str:
        auth_params = dict(
            client_id=self.client_id,
            redirect_uri=self.redirect_uri,
            scope=" ".join(scope),
            response_type=response_type
        )
        if instance_name:
            auth_params["instance_name"] = instance_name

        response = self.client.post(AUTH_URL, params=auth_params)
        return str(response.url)

    def get_access_token(
            self,
            code: str,
            grant_type: Literal["authorization_code"] = "authorization_code",
            client_secret: str | None = None
    ) -> str | None:
        token_params = dict(
            code=code,
            client_id=self.client_id,
            redirect_uri=self.redirect_uri,
            grant_type=grant_type
        )
        if client_secret:
            token_params["client_secret"] = client_secret
        response = self.client.post(TOKEN_URL, params=token_params)
        return self._process_get_access_token(response)


class AuthorizationAsync(_BaseAuthorization):
    def __init__(self, client_id: str, redirect_uri: str):
        super().__init__(client_id, redirect_uri)
        self.client = AsyncClient(
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            }
        )

    async def close(self):
        await self.client.aclose()

    async def authorization_request(
            self,
            response_type: Literal["code"] = "code",
            scope: list[str] = DEFAULT_SCOPE,
            instance_name: str | None = None
    ) -> str:
        auth_params = dict(
            client_id=self.client_id,
            redirect_uri=self.redirect_uri,
            scope=" ".join(scope),
            response_type=response_type
        )
        if instance_name:
            auth_params["instance_name"] = instance_name

        response = await self.client.post(AUTH_URL, params=auth_params)
        return str(response.url)

    async def get_access_token(
            self,
            code: str,
            grant_type: Literal["authorization_code"] = "authorization_code",
            client_secret: str | None = None
    ) -> str | None:
        token_params = dict(
            code=code,
            client_id=self.client_id,
            redirect_uri=self.redirect_uri,
            grant_type=grant_type
        )
        if client_secret:
            token_params["client_secret"] = client_secret
        response = await self.client.post(TOKEN_URL, params=token_params)
        return self._process_get_access_token(response)



from .base import Authorization, DEFAULT_SCOPE
from ayoomoney.errors import CreateTokenError


def authorize(client_id: str, redirect_uri: str, scope: list[str] = DEFAULT_SCOPE, *_):
    access_token = ""
    auth = Authorization(client_id, redirect_uri)
    try:
        url = auth.authorization_request(scope=scope)

        print(f"Перейдите по URL и подтвердите доступ для приложения\n{url}")
        code = input("Введите код из адресной строки в консоль >>>  ").strip()

        access_token = auth.get_access_token(code)
    except CreateTokenError as e:
        print(e)
    finally:
        auth.close()

    if not access_token:
        return

    print(f"Ваш токен — {access_token}. Сохраните его в безопасном месте!")
    return
